@@ -1,7 +1,6 @@
 /* Buffer management for tar.
 
-   Copyright 1988, 1992-1994, 1996-1997, 1999-2010, 2013 Free Software
-   Foundation, Inc.
+   Copyright 1988-2019 Free Software Foundation, Inc.
 
    This file is part of GNU tar.
 
@@ -105,13 +104,20 @@ bool write_archive_to_stdout;
 
 /* When creating a multi-volume archive, each 'bufmap' represents
    a member stored (perhaps partly) in the current record buffer.
+   Bufmaps are form a single-linked list in chronological order.
+
    After flushing the record to the output media, all bufmaps that
-   represent fully written members are removed from the list, then
-   the sizeleft and start numbers in the remaining bufmaps are updated.
+   represent fully written members are removed from the list, the
+   nblocks and sizeleft values in the bufmap_head and start values
+   in all remaining bufmaps are updated.  The information stored
+   in bufmap_head is used to form the volume header.
 
    When reading from a multi-volume archive, the list degrades to a
    single element, which keeps information about the member currently
-   being read.
+   being read.  In that case the sizeleft member is updated explicitly
+   from the extractor code by calling the mv_size_left function.  The
+   information from bufmap_head is compared with the volume header data
+   to ensure that subsequent volumes are fed in the right order.
 */
 
 struct bufmap
@@ -121,6 +127,7 @@ struct bufmap
   char *file_name;              /* Name of the stored file */
   off_t sizetotal;              /* Size of the stored file */
   off_t sizeleft;               /* Size left to read/write */
+  size_t nblocks;               /* Number of blocks written since reset */
 };
 static struct bufmap *bufmap_head, *bufmap_tail;
 
@@ -145,6 +152,7 @@ mv_begin_write (const char *file_name, off_t totsize, off_t sizeleft)
       bp->file_name = xstrdup (file_name);
       bp->sizetotal = totsize;
       bp->sizeleft = sizeleft;
+      bp->nblocks = 0;
     }
 }
 
@@ -155,8 +163,7 @@ bufmap_locate (size_t off)
 
   for (map = bufmap_head; map; map = map->next)
     {
-      if (!map->next
-	  || off < map->next->start * BLOCKSIZE)
+      if (!map->next || off < map->next->start * BLOCKSIZE)
 	break;
     }
   return map;
@@ -185,7 +192,10 @@ bufmap_reset (struct bufmap *map, ssize_t fixup)
   if (map)
     {
       for (; map; map = map->next)
-	map->start += fixup;
+	{
+	  map->start += fixup;
+	  map->nblocks = 0;
+	}
     }
 }
 
@@ -247,7 +257,7 @@ set_volume_start_time (void)
   last_stat_time = volume_start_time;
 }
 
-void
+double
 compute_duration (void)
 {
   struct timespec now;
@@ -255,6 +265,7 @@ compute_duration (void)
   duration += ((now.tv_sec - last_stat_time.tv_sec)
                + (now.tv_nsec - last_stat_time.tv_nsec) / 1e9);
   gettime (&last_stat_time);
+  return duration;
 }
 
 
@@ -269,7 +280,8 @@ enum compress_type {
   ct_lzip,
   ct_lzma,
   ct_lzop,
-  ct_xz
+  ct_xz,
+  ct_zstd
 };
 
 static enum compress_type archive_compression_type = ct_none;
@@ -298,6 +310,7 @@ static struct zip_magic const magic[] = {
   { ct_lzma,     6, "\xFFLZMA" },
   { ct_lzop,     4, "\211LZO" },
   { ct_xz,       6, "\xFD" "7zXZ" },
+  { ct_zstd,     4, "\x28\xB5\x2F\xFD" },
 };
 
 #define NMAGIC (sizeof(magic)/sizeof(magic[0]))
@@ -313,6 +326,7 @@ static struct zip_program zip_program[] = {
   { ct_lzma,     XZ_PROGRAM,       "-J" },
   { ct_lzop,     LZOP_PROGRAM,     "--lzop" },
   { ct_xz,       XZ_PROGRAM,       "-J" },
+  { ct_zstd,     ZSTD_PROGRAM,     "--zstd" },
   { ct_none }
 };
 
@@ -390,7 +404,12 @@ check_compressed_archive (bool *pshort)
   /* Restore global values */
   read_full_records = sfr;
 
-  if (tar_checksum (record_start, true) == HEADER_SUCCESS)
+  if (record_start != record_end /* no files smaller than BLOCKSIZE */
+      && (strcmp (record_start->header.magic, TMAGIC) == 0
+          || strcmp (record_start->buffer + offsetof (struct posix_header,
+                                                      magic),
+                     OLDGNU_MAGIC) == 0)
+      && tar_checksum (record_start, true) == HEADER_SUCCESS)
     /* Probably a valid header */
     return ct_tar;
 
@@ -488,64 +507,98 @@ open_compressed_archive (void)
   return archive;
 }
 
-
-static void
+static int
 print_stats (FILE *fp, const char *text, tarlong numbytes)
 {
-  char bytes[sizeof (tarlong) * CHAR_BIT];
   char abbr[LONGEST_HUMAN_READABLE + 1];
   char rate[LONGEST_HUMAN_READABLE + 1];
+  int n = 0;
 
   int human_opts = human_autoscale | human_base_1024 | human_SI | human_B;
 
-  sprintf (bytes, TARLONG_FORMAT, numbytes);
-
-  fprintf (fp, "%s: %s (%s, %s/s)\n",
-           text, bytes,
-           human_readable (numbytes, abbr, human_opts, 1, 1),
-           (0 < duration && numbytes / duration < (uintmax_t) -1
-            ? human_readable (numbytes / duration, rate, human_opts, 1, 1)
-            : "?"));
+  if (text && text[0])
+    n += fprintf (fp, "%s: ", gettext (text));
+  return n + fprintf (fp, TARLONG_FORMAT " (%s, %s/s)",
+		      numbytes,
+		      human_readable (numbytes, abbr, human_opts, 1, 1),
+		      (0 < duration && numbytes / duration < (uintmax_t) -1
+		       ? human_readable (numbytes / duration, rate, human_opts, 1, 1)
+		       : "?"));
 }
 
-void
-print_total_stats (void)
+/* Format totals to file FP.  FORMATS is an array of strings to output
+   before each data item (bytes read, written, deleted, in that order).
+   EOR is a delimiter to output after each item (used only if deleting
+   from the archive), EOL is a delimiter to add at the end of the output
+   line. */
+int
+format_total_stats (FILE *fp, char const *const *formats, int eor, int eol)
 {
+  int n;
+
   switch (subcommand_option)
     {
     case CREATE_SUBCOMMAND:
     case CAT_SUBCOMMAND:
     case UPDATE_SUBCOMMAND:
     case APPEND_SUBCOMMAND:
-      /* Amanda 2.4.1p1 looks for "Total bytes written: [0-9][0-9]*".  */
-      print_stats (stderr, _("Total bytes written"),
-                   prev_written + bytes_written);
+      n = print_stats (fp, formats[TF_WRITE],
+		       prev_written + bytes_written);
       break;
 
     case DELETE_SUBCOMMAND:
       {
         char buf[UINTMAX_STRSIZE_BOUND];
-        print_stats (stderr, _("Total bytes read"),
-                     records_read * record_size);
-        print_stats (stderr, _("Total bytes written"),
-                     prev_written + bytes_written);
-        fprintf (stderr, _("Total bytes deleted: %s\n"),
-                 STRINGIFY_BIGINT ((records_read - records_skipped)
-                                    * record_size
-                                   - (prev_written + bytes_written), buf));
+        n = print_stats (fp, formats[TF_READ],
+			 records_read * record_size);
+
+	fputc (eor, fp);
+	n++;
+
+        n += print_stats (fp, formats[TF_WRITE],
+			  prev_written + bytes_written);
+
+	fputc (eor, fp);
+	n++;
+
+	if (formats[TF_DELETED] && formats[TF_DELETED][0])
+	  n += fprintf (fp, "%s: ", gettext (formats[TF_DELETED]));
+        n += fprintf (fp, "%s",
+		      STRINGIFY_BIGINT ((records_read - records_skipped)
+					* record_size
+					- (prev_written + bytes_written), buf));
       }
       break;
 
     case EXTRACT_SUBCOMMAND:
     case LIST_SUBCOMMAND:
     case DIFF_SUBCOMMAND:
-      print_stats (stderr, _("Total bytes read"),
-                   records_read * record_size);
+      n = print_stats (fp, _(formats[TF_READ]),
+		       records_read * record_size);
       break;
 
     default:
       abort ();
     }
+  if (eol)
+    {
+      fputc (eol, fp);
+      n++;
+    }
+  return n;
+}
+
+static char const *const default_total_format[] = {
+  N_("Total bytes read"),
+  /* Amanda 2.4.1p1 looks for "Total bytes written: [0-9][0-9]*".  */
+  N_("Total bytes written"),
+  N_("Total bytes deleted")
+};
+
+void
+print_total_stats (void)
+{
+  format_total_stats (stderr, default_total_format, '\n', '\n');
 }
 
 /* Compute and return the block ordinal at current_block.  */
@@ -633,6 +686,22 @@ init_buffer (void)
   record_end = record_start + blocking_factor;
 }
 
+static void
+check_tty (enum access_mode mode)
+{
+  /* Refuse to read archive from and write it to a tty. */
+  if (strcmp (archive_name_array[0], "-") == 0
+      && isatty (mode == ACCESS_READ ? STDIN_FILENO : STDOUT_FILENO))
+    {
+      FATAL_ERROR ((0, 0,
+		    mode == ACCESS_READ
+		    ? _("Refusing to read archive contents from terminal "
+			"(missing -f option?)")
+		    : _("Refusing to write archive contents to terminal "
+			"(missing -f option?)")));
+    }
+}
+
 /* Open an archive file.  The argument specifies whether we are
    reading or writing, or both.  */
 static void
@@ -653,6 +722,7 @@ _open_archive (enum access_mode wanted_access)
 
   /* When updating the archive, we start with reading.  */
   access_mode = wanted_access == ACCESS_UPDATE ? ACCESS_READ : wanted_access;
+  check_tty (access_mode);
 
   read_full_records = read_full_records_option;
 
@@ -696,8 +766,7 @@ _open_archive (enum access_mode wanted_access)
             enum compress_type type;
 
             archive = STDIN_FILENO;
-	    freopen (NULL, "rb", stdin);
-
+            freopen (NULL, "rb", stdin);
             type = check_compressed_archive (&shortfile);
             if (type != ct_tar && type != ct_none)
               FATAL_ERROR ((0, 0,
@@ -710,19 +779,19 @@ _open_archive (enum access_mode wanted_access)
 
         case ACCESS_WRITE:
           archive = STDOUT_FILENO;
-	  freopen (NULL,
-		   fcntl (STDOUT_FILENO, F_GETFL) & O_APPEND ? "ab" : "wb",
-		   stdout);
+          freopen (NULL,
+                   fcntl (STDOUT_FILENO, F_GETFL) & O_APPEND ? "ab" : "wb",
+                   stdout);
           if (!index_file_name)
             stdlis = stderr;
           break;
 
         case ACCESS_UPDATE:
           archive = STDIN_FILENO;
-	  freopen (NULL, "rb", stdin);
-	  freopen (NULL,
-		   fcntl (STDOUT_FILENO, F_GETFL) & O_APPEND ? "ab" : "wb",
-		   stdout);
+          freopen (NULL, "rb", stdin);
+          freopen (NULL,
+                   fcntl (STDOUT_FILENO, F_GETFL) & O_APPEND ? "ab" : "wb",
+                   stdout);
           write_archive_to_stdout = true;
           record_end = record_start; /* set up for 1st record = # 0 */
           if (!index_file_name)
@@ -822,12 +891,19 @@ _flush_write (void)
       if (map)
 	{
 	  size_t delta = status - map->start * BLOCKSIZE;
+	  ssize_t diff;
+	  map->nblocks += delta / BLOCKSIZE;
 	  if (delta > map->sizeleft)
 	    delta = map->sizeleft;
 	  map->sizeleft -= delta;
 	  if (map->sizeleft == 0)
-	    map = map->next;
-	  bufmap_reset (map, map ? (- map->start) : 0);
+	    {
+	      diff = map->start + map->nblocks;
+	      map = map->next;
+	    }
+	  else
+	    diff = map->start;
+	  bufmap_reset (map, - diff);
 	}
     }
   return status;
@@ -937,18 +1013,28 @@ short_read (size_t status)
 void
 flush_archive (void)
 {
-  size_t buffer_level = current_block->buffer - record_start->buffer;
-  record_start_block += record_end - record_start;
-  current_block = record_start;
-  record_end = record_start + blocking_factor;
+  size_t buffer_level;
 
   if (access_mode == ACCESS_READ && time_to_start_writing)
     {
       access_mode = ACCESS_WRITE;
       time_to_start_writing = false;
       backspace_output ();
+      if (record_end - record_start < blocking_factor)
+	{
+	  memset (record_end, 0,
+		  (blocking_factor - (record_end - record_start))
+		  * BLOCKSIZE);
+	  record_end = record_start + blocking_factor;
+	  return;
+	}
     }
 
+  buffer_level = current_block->buffer - record_start->buffer;
+  record_start_block += record_end - record_start;
+  current_block = record_start;
+  record_end = record_start + blocking_factor;
+
   switch (access_mode)
     {
     case ACCESS_READ:
@@ -988,7 +1074,7 @@ backspace_output (void)
 
     /* Seek back to the beginning of this record and start writing there.  */
 
-    position -= record_size;
+    position -= record_end->buffer - record_start->buffer;
     if (position < 0)
       position = 0;
     if (rmtlseek (archive, position, SEEK_SET) != position)
@@ -1049,9 +1135,9 @@ close_archive (void)
 {
   if (time_to_start_writing || access_mode == ACCESS_WRITE)
     {
-      flush_archive ();
-      if (current_block > record_start)
-        flush_archive ();
+      do
+	flush_archive ();
+      while (current_block > record_start);
     }
 
   compute_duration ();
@@ -1069,6 +1155,16 @@ close_archive (void)
   bufmap_free (NULL);
 }
 
+void
+write_fatal_details (char const *name, ssize_t status, size_t size)
+{
+  write_error_details (name, status, size);
+  if (rmtclose (archive) != 0)
+    close_error (*archive_name_cursor);
+  sys_wait_for_child (child_pid, false);
+  fatal_exit ();
+}
+
 /* Called to initialize the global volume number.  */
 void
 init_volume_number (void)
@@ -1210,8 +1306,7 @@ change_tape_menu (FILE *read_file)
               sys_spawn_shell ();
               break;
             }
-          /* FALL THROUGH */
-
+	  FALLTHROUGH;
         default:
           fprintf (stderr, _("Invalid input. Type ? for help.\n"));
         }
@@ -1363,7 +1458,10 @@ try_new_volume (void)
 
   header = find_next_block ();
   if (!header)
-    return false;
+    {
+      WARN ((0, 0, _("This does not look like a tar archive")));
+      return false;
+    }
 
   switch (header->header.typeflag)
     {
@@ -1373,7 +1471,7 @@ try_new_volume (void)
 	if (read_header (&header, &dummy, read_header_x_global)
 	    != HEADER_SUCCESS_EXTENDED)
 	  {
-	    ERROR ((0, 0, _("This does not look like a tar archive")));
+	    WARN ((0, 0, _("This does not look like a tar archive")));
 	    return false;
 	  }
 
@@ -1402,7 +1500,7 @@ try_new_volume (void)
 	    break;
 
 	  default:
-	    ERROR ((0, 0, _("This does not look like a tar archive")));
+	    WARN ((0, 0, _("This does not look like a tar archive")));
 	    return false;
 	  }
         break;
@@ -1412,18 +1510,17 @@ try_new_volume (void)
       if (!read_header0 (&dummy))
         return false;
       tar_stat_destroy (&dummy);
-      assign_string (&volume_label, current_header->header.name);
+      ASSIGN_STRING_N (&volume_label, current_header->header.name);
       set_next_block_after (header);
       header = find_next_block ();
       if (header->header.typeflag != GNUTYPE_MULTIVOL)
         break;
-      /* FALL THROUGH */
-
+      FALLTHROUGH;
     case GNUTYPE_MULTIVOL:
       if (!read_header0 (&dummy))
         return false;
       tar_stat_destroy (&dummy);
-      assign_string (&continued_file_name, current_header->header.name);
+      ASSIGN_STRING_N (&continued_file_name, current_header->header.name);
       continued_file_size =
         UINTMAX_FROM_HEADER (current_header->header.size);
       continued_file_offset =
@@ -1437,8 +1534,14 @@ try_new_volume (void)
   if (bufmap_head)
     {
       uintmax_t s;
-      if (!continued_file_name
-          || strcmp (continued_file_name, bufmap_head->file_name))
+      if (!continued_file_name)
+	{
+	  WARN ((0, 0, _("%s is not continued on this volume"),
+		 quote (bufmap_head->file_name)));
+	  return false;
+	}
+
+      if (strcmp (continued_file_name, bufmap_head->file_name))
         {
           if ((archive_format == GNU_FORMAT || archive_format == OLDGNU_FORMAT)
               && strlen (bufmap_head->file_name) >= NAME_FIELD_SIZE
@@ -1560,15 +1663,7 @@ match_volume_label (void)
 		      quote (volume_label_option)));
       if (label->header.typeflag == GNUTYPE_VOLHDR)
 	{
-	  if (memchr (label->header.name, '\0', sizeof label->header.name))
-	    assign_string (&volume_label, label->header.name);
-	  else
-	    {
-	      volume_label = xmalloc (sizeof (label->header.name) + 1);
-	      memcpy (volume_label, label->header.name,
-		      sizeof (label->header.name));
-	      volume_label[sizeof (label->header.name)] = 0;
-	    }
+	  ASSIGN_STRING_N (&volume_label, label->header.name);
 	}
       else if (label->header.typeflag == XGLTYPE)
 	{
@@ -1604,8 +1699,7 @@ _write_volume_label (const char *str)
       memset (label, 0, BLOCKSIZE);
 
       strcpy (label->header.name, str);
-      assign_string (&current_stat_info.file_name,
-                     label->header.name);
+      assign_string (&current_stat_info.file_name, label->header.name);
       current_stat_info.had_trailing_slash =
         strip_trailing_slashes (current_stat_info.file_name);
 
@@ -1636,7 +1730,6 @@ add_chunk_header (struct bufmap *map)
 {
   if (archive_format == POSIX_FORMAT)
     {
-      off_t block_ordinal;
       union block *blk;
       struct tar_stat_info st;
 
@@ -1651,11 +1744,10 @@ add_chunk_header (struct bufmap *map)
       st.file_name = st.orig_file_name;
       st.archive_file_size = st.stat.st_size = map->sizeleft;
 
-      block_ordinal = current_block_ordinal ();
       blk = start_header (&st);
       if (!blk)
         abort (); /* FIXME */
-      finish_header (&st, blk, block_ordinal);
+      simple_finish_header (write_extended (false, &st, blk));
       free (st.orig_file_name);
     }
 }
@@ -1677,15 +1769,19 @@ gnu_add_multi_volume_header (struct bufmap *map)
 {
   int tmp;
   union block *block = find_next_block ();
+  size_t len = strlen (map->file_name);
 
-  if (strlen (map->file_name) > NAME_FIELD_SIZE)
-    WARN ((0, 0,
-           _("%s: file name too long to be stored in a GNU multivolume header, truncated"),
-           quotearg_colon (map->file_name)));
+  if (len > NAME_FIELD_SIZE)
+    {
+      WARN ((0, 0,
+	     _("%s: file name too long to be stored in a GNU multivolume header, truncated"),
+	     quotearg_colon (map->file_name)));
+      len = NAME_FIELD_SIZE;
+    }
 
   memset (block, 0, BLOCKSIZE);
 
-  strncpy (block->header.name, map->file_name, NAME_FIELD_SIZE);
+  memcpy (block->header.name, map->file_name, len);
   block->header.typeflag = GNUTYPE_MULTIVOL;
 
   OFF_TO_CHARS (map->sizeleft, block->header.size);
